<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

/**
 * Abstract base class for filtering operations. Objects of children of this
 * class get input of a specific type and produce output of a specific type.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-05
 */
abstract class Filter {
	/**
	 * Default constructor. Child classes have to call it to set data types.
	 *
	 * @param $inType Type for incoming data.
	 * @param $outType Type for outgoing data.
	 */
	protected function __construct($inType, $outType) {
		$this->inDataType = $inType;
		$this->outDataType = $outType;
		$this->sinks = array();
	}

	/**
	 * Consume incoming data. This function checks the type of the data and
	 * in case of correct type delegates the
	 *
	 * @param $data Incoming data.
	 */
	final public function receive(& $data) {
		if($data instanceof $this->inDataType) {
			$this->timeStart = microtime(TRUE);
			$this->memStart = memory_get_usage();
			$this->process($data);
		} else {
			throw new Exception('Input data is of wrong type.');
		}
	}

	/**
	 * Do something with the data. This function has to be implemented by child
	 * classes. It gets the incoming data and should produce some output by
	 * calling the @b send() function
	 */
	protected abstract function process(& $data);

	/**
	 * Send the data produced by this filter to all sinks.
	 *
	 * @param $data Data which should be sent.
	 */
	final protected function send(& $data) {
		$this->timeEnd = microtime(TRUE);
		$this->memEnd = memory_get_usage();
		if(!($data instanceof $this->outDataType)) {
			throw new Exception('Output data is of wrong type.');
		}
		foreach($this->sinks as $sink) {
			$sink->receive($data);
		}
	}

	/**
	 * Add a new sink to this filter.
	 *
	 * @param $sink Additional sink.
	 */
	final public function attachSink(Filter & $sink) {
		if($this->outDataType !== $sink->inDataType) {
			throw new Exception('Sink is incompatible.');
		}
		$this->sinks[] = $sink;
	}

	/**
	 * Return the time it took to process the data.
	 *
	 * @return Number of seconds as floating point value.
	 */
	final public function getProcessTime() {
		return ($this->timeEnd - $this->timeStart);
	}

	/**
	 * Return the amount of memory which was allocated and not freed.
	 *
	 * @return Number of bytes.
	 */
	final public function getMemoryUsage() {
		return ($this->memEnd - $this->memStart);
	}

	/**
	 * Type of incoming data.
	 */
	private $inDataType;

	/**
	 * Type of outgoing data.
	 */
	private $outDataType;

	/**
	 * List of sinks which will be fed with data leaving this filter.
	 */
	private $sinks;

	/**
	 * Stores time at which processing begins.
	 */
	private $timeStart;

	/**
	 * Stores time at which processing ends.
	 */
	private $timeEnd;

	/**
	 * Stores amount of memory at time at which processing begins.
	 */
	private $memStart;

	/**
	 * Stores amount of memory at time at which processing ends.
	 */
	private $memEnd;
}
?>
