<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');
require_once('MathVector.php');

/**
 * Class for loading Wavefront Object (.obj) files.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-10-29
 * @see http://people.scs.fsu.edu/~burkardt/txt/obj_format.txt
 */
class InputObj extends Filter {

	/**
	 * This class accepts a File object as input and generates a Data3D object 
	 * as output.
	 */
	public function __construct() {
		parent::__construct('File', 'Data3D');
	}

	/**
	 * Read scene data from a .obj file.
	 *
	 * @param $data File object used to get the path to the file.
	 */
	public function process(& $data) {
		$scene = new Data3D;
		$vertices = array();
		$normals = array();
		$texCoords = array();
		$material = null;
		$line = 1;
		$file = fopen($data->getPath(), 'rb');
		if($file === FALSE) {
			throw new Exception('Cannot open file.');
		}
		while(!feof($file)) {
			$input = fgets($file);
			if(strncmp($input, 'vn', 2) == 0) {
				$floats = array(0.0, 0.0, 0.0);
				$found = sscanf($input, '%s %f %f %f ', $unused, $floats[0], $floats[1], $floats[2]);
				if($found != 4) {
					throw new Exception('Wrong usage of vn on line '.$line.'.');
				}
				$normals[] = new Vector4($floats[0], $floats[1], $floats[2], 0.0);
			} else if(strncmp($input, 'vt', 2) == 0) {
				$floats = array(0.0, 0.0, 0.0);
				$found = sscanf($input, '%s %f %f %f %f ', $unused, $floats[0], $floats[1], $floats[2], $floats[3]);
				if($found < 2) {
					throw new Exception('Wrong usage of vt on line '.$line.'.');
				}
				$texCoords[] = new Vector4($floats[0], $floats[1], $floats[2], $floats[3]);
			} else if(strncmp($input, 'v', 1) == 0) {
				$floats = array(0.0, 0.0, 0.0, 1.0);
				$found = sscanf($input, '%s %f %f %f %f ', $unused, $floats[0], $floats[1], $floats[2], $floats[3]);
				if($found < 4) {
					throw new Exception('Wrong usage of v on line '.$line.'.');
				}
				$vertices[] = new Vector4($floats[0], $floats[1], $floats[2], $floats[3]);
			} else if(strncmp($input, 'f', 1) == 0) {
				$parts = explode(' ', $input, 5);
				$elements = count($parts);
				if($elements == 4 || $elements == 5) {
					// Loop with offset for triangulation.
					for($offset = 1; $offset < $elements - 1; $offset += 2) {
						$face = new Face;
						for($i = $offset; $i < 3 + $offset; $i++) {
							$values = explode('/', $parts[$i == $elements ? 1 : $i], 3);
							if(!isset($values[0]) || $values[0] == 0) {
								throw new Exception('Vertex not set for face on line '.$line.'.');
							}
							$face->vertices[$i - $offset] =& $vertices[$values[0] - 1];
							if(isset($values[1]) && $values[1] > 0) {
								$face->texCoords[$i - $offset] =& $texCoords[$values[1] - 1];
							}
							if(isset($values[2]) && $values[2] > 0) {
								$face->normals[$i - $offset] =& $normals[$values[2] - 1];
							}
						}
						$face->material =& $material;
						$scene->faces[] = $face;
					}
				} else {
					throw new Exception('Wrong usage of f on line '.$line.'.');
				}
			} else if(strncmp($input, 'mtllib', 6) == 0) {
				$this->loadMaterials(trim(substr($input, 7)));
			} else if(strncmp($input, 'usemtl', 6) == 0) {
				$materialName = trim(substr($input, 7));
				if($materialName === '(null)') {
					throw new Exception('Models without materials are not supported (material (null) on line '.$line.').');
				}
				if(!isset($this->materials[$materialName])) {
					throw new Exception('Usage of unknown material on line '.$line.'.');
				}
				$material = $this->materials[$materialName];
			}
			$line++;
		}
		fclose($file);
		unset($vertices);
		unset($normals);
		unset($texCoords);
		$this->send($scene);
	}

	/**
	 * Read material data from a .mtl file.
	 *
	 * @param $mtlFilePath File path of the material file.
	 */
	protected function loadMaterials($mtlFilePath) {
		$line = 1;
		$mtlFile = fopen($mtlFilePath, 'rb');
		if($mtlFile === FALSE) {
			throw new Exception('Cannot open file.');
		}
		while(!feof($mtlFile)) {
			$input = fgets($mtlFile);
			if(strncmp($input, 'newmtl', 6) == 0) {
				$currentMaterial = trim(substr($input, 7));
				$this->materials[$currentMaterial] = new Material;
			} else if(strncmp($input, 'Ns', 2) == 0) {
				$floats = 0.0;
				$found = sscanf($input, '%s %f ', $unused, $float);
				if($found != 2) {
					throw new Exception('Wrong usage of Ns on line '.$line.'.');
				}
				$this->materials[$currentMaterial]->shininess = $float;
			} else if(strncmp($input, 'Ka', 2) == 0) {
				$floats = array(0.0, 0.0, 0.0);
				$found = sscanf($input, '%s %f %f %f ', $unused, $floats[0], $floats[1], $floats[2]);
				if($found != 4) {
					throw new Exception('Wrong usage of Ka on line '.$line.'.');
				}
				$this->materials[$currentMaterial]->ambient = new Vector4($floats[0], $floats[1], $floats[2], 0.0);
			} else if(strncmp($input, 'Kd', 2) == 0) {
				$floats = array(0.0, 0.0, 0.0);
				$found = sscanf($input, '%s %f %f %f ', $unused, $floats[0], $floats[1], $floats[2]);
				if($found != 4) {
					throw new Exception('Wrong usage of Kd on line '.$line.'.');
				}
				$this->materials[$currentMaterial]->diffuse = new Vector4($floats[0], $floats[1], $floats[2], 0.0);
			} else if(strncmp($input, 'Ks', 2) == 0) {
				$floats = array(0.0, 0.0, 0.0);
				$found = sscanf($input, '%s %f %f %f ', $unused, $floats[0], $floats[1], $floats[2]);
				if($found != 4) {
					throw new Exception('Wrong usage of Ks on line '.$line.'.');
				}
				$this->materials[$currentMaterial]->specular = new Vector4($floats[0], $floats[1], $floats[2], 0.0);
			}
			$line++;
		}
		fclose($mtlFile);
	}

	/**
	 * Array holding all known materials of type @b Material.
	 */
	protected $materials;
}
?>
