<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('MathVector.php');

/**
 * Class representing a 4 × 4 matrix and operations on it.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-10-31, 2008-12-11
 */
class Matrix4x4 {
	/**
	 * Default constructor setting the matrix to the identity matrix.
	 */
	function __construct() {
		$this->makeIdentity();
	}

	/**
	 * Set the data of the matrix to be the identity matrix.
	 */
	function makeIdentity() {
		for($i = 0; $i < 16; $i++) {
			if($i % 5 == 0) {
				$this->data[$i] = 1.0;
			} else {
				$this->data[$i] = 0.0;
			}
		}
	}

	/**
	 * Multiply two matrices. Fast matrix times matrix multiplication. The 
	 * indices are “hard-coded” and therefore there are no loops.
	 *
	 * @param $m 4 × 4 matrix (with 16 elements in column-major order).
	 * @return Resulting matrix with $result = $this · $m.
	 */
	function mult(Matrix4x4 $m) {
		$result = new Matrix4x4;

		$result->data[ 0] = $m->data[ 0] * $this->data[ 0] + $m->data[ 4] * $this->data[ 1] + $m->data[ 8] * $this->data[ 2] + $m->data[12] * $this->data[ 3];
		$result->data[ 1] = $m->data[ 1] * $this->data[ 0] + $m->data[ 5] * $this->data[ 1] + $m->data[ 9] * $this->data[ 2] + $m->data[13] * $this->data[ 3];
		$result->data[ 2] = $m->data[ 2] * $this->data[ 0] + $m->data[ 6] * $this->data[ 1] + $m->data[10] * $this->data[ 2] + $m->data[14] * $this->data[ 3];
		$result->data[ 3] = $m->data[ 3] * $this->data[ 0] + $m->data[ 7] * $this->data[ 1] + $m->data[11] * $this->data[ 2] + $m->data[15] * $this->data[ 3];
	
		$result->data[ 4] = $m->data[ 0] * $this->data[ 4] + $m->data[ 4] * $this->data[ 5] + $m->data[ 8] * $this->data[ 6] + $m->data[12] * $this->data[ 7];
		$result->data[ 5] = $m->data[ 1] * $this->data[ 4] + $m->data[ 5] * $this->data[ 5] + $m->data[ 9] * $this->data[ 6] + $m->data[13] * $this->data[ 7];
		$result->data[ 6] = $m->data[ 2] * $this->data[ 4] + $m->data[ 6] * $this->data[ 5] + $m->data[10] * $this->data[ 6] + $m->data[14] * $this->data[ 7];
		$result->data[ 7] = $m->data[ 3] * $this->data[ 4] + $m->data[ 7] * $this->data[ 5] + $m->data[11] * $this->data[ 6] + $m->data[15] * $this->data[ 7];
	
		$result->data[ 8] = $m->data[ 0] * $this->data[ 8] + $m->data[ 4] * $this->data[ 9] + $m->data[ 8] * $this->data[10] + $m->data[12] * $this->data[11];
		$result->data[ 9] = $m->data[ 1] * $this->data[ 8] + $m->data[ 5] * $this->data[ 9] + $m->data[ 9] * $this->data[10] + $m->data[13] * $this->data[11];
		$result->data[10] = $m->data[ 2] * $this->data[ 8] + $m->data[ 6] * $this->data[ 9] + $m->data[10] * $this->data[10] + $m->data[14] * $this->data[11];
		$result->data[11] = $m->data[ 3] * $this->data[ 8] + $m->data[ 7] * $this->data[ 9] + $m->data[11] * $this->data[10] + $m->data[15] * $this->data[11];
	
		$result->data[12] = $m->data[ 0] * $this->data[12] + $m->data[ 4] * $this->data[13] + $m->data[ 8] * $this->data[14] + $m->data[12] * $this->data[15];
		$result->data[13] = $m->data[ 1] * $this->data[12] + $m->data[ 5] * $this->data[13] + $m->data[ 9] * $this->data[14] + $m->data[13] * $this->data[15];
		$result->data[14] = $m->data[ 2] * $this->data[12] + $m->data[ 6] * $this->data[13] + $m->data[10] * $this->data[14] + $m->data[14] * $this->data[15];
		$result->data[15] = $m->data[ 3] * $this->data[12] + $m->data[ 7] * $this->data[13] + $m->data[11] * $this->data[14] + $m->data[15] * $this->data[15];

		return $result;
	}

	/**
	 * Multiply a vector with this matrix. Fast matrix times vector 
	 * multiplication. The indices are “hard-coded” and therefore there are no 
	 * loops.
	 *
	 * @param $v Four-dimensional vector.
	 * @return Resulting four-dimensional vector with $result = $this · $v.
	 */
	function multVector(Vector4 $v) {
		$vector = $v->getArray();

		$result[ 0] = $this->data[ 0] * $vector[0] + $this->data[ 1] * $vector[1] + $this->data[ 2] * $vector[2] + $this->data[ 3] * $vector[3];
		$result[ 1] = $this->data[ 4] * $vector[0] + $this->data[ 5] * $vector[1] + $this->data[ 6] * $vector[2] + $this->data[ 7] * $vector[3];
		$result[ 2] = $this->data[ 8] * $vector[0] + $this->data[ 9] * $vector[1] + $this->data[10] * $vector[2] + $this->data[11] * $vector[3];
		$result[ 3] = $this->data[12] * $vector[0] + $this->data[13] * $vector[1] + $this->data[14] * $vector[2] + $this->data[15] * $vector[3];

		return new Vector4($result[0], $result[1], $result[2], $result[3]);
	}

	/**
	 * Return the matrix as human-readable string representation.
	 *
	 * @return Matrix representation as string.
	 */
	function __toString() {
		$result = '';
		for($i = 0; $i < 16; $i++) {
			if($i == 0) {
				$result .= "/\t";
			} else if($i == 4 || $i == 8) {
				$result .= "|\t";
			} else if($i == 12) {
				$result .= "\\\t";
			}
			
			$result .= $this->data[$i]."\t";
			
			if($i == 3) {
				$result .= "\\\n";
			} else if($i == 7 || $i == 11) {
				$result .= "|\n";
			} else if($i == 15) {
				$result .= "/\n";
			}
		}
		return $result;
	}

	/**
	 * Array storing the entries of the matrix. The data is stored as array with
	 * length four (rows) of arrays with length four (columns). This means it is
	 * row-major.
	 */
	protected $data;
}
?>
