<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

/**
 * Class representing a four-dimensional vector.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-01
 */
class Vector4 {
	/**
	 * Default constructor setting the initial values of the vector.
	 *
	 * @param $x First coordinate.
	 * @param $y Second coordinate.
	 * @param $z Third coordinate.
	 * @param $w Fourth (homogeneous) coordinate.
	 */
	function __construct($x = 0.0, $y = 0.0, $z = 0.0, $w = 0.0) {
		$this->x = $x;
		$this->y = $y;
		$this->z = $z;
		$this->w = $w;
	}

	/**
	 * Set the values of this vector.
	 *
	 * @param $x First coordinate.
	 * @param $y Second coordinate.
	 * @param $z Third coordinate.
	 * @param $w Fourth (homogeneous) coordinate.
	 */
	function set($x, $y, $z, $w) {
		$this->x = $x;
		$this->y = $y;
		$this->z = $z;
		$this->w = $w;
	}

	/**
	 * Set the values of this vector given by an array.
	 *
	 * @param $a Array with three or four elements.
	 */
	function setArray($a) {
		if(count($a) == 3) {
			$this->x = $a[0];
			$this->y = $a[1];
			$this->z = $a[2];
			$this->w = 1.0;
		} else if(count($a) == 4) {
			$this->x = $a[0];
			$this->y = $a[1];
			$this->z = $a[2];
			$this->w = $a[3];
		} else {
			throw new Exception('Invalid argument.');
		}
	}

	/**
	 * Return a coordinate of the vector.
	 *
	 * @param $index Index of the coordinate in interval [0; 3].
	 * @return Requested value of the vector.
	 */
	function get($index) {
		switch($index) {
			case 0:
				return $this->x;
			case 1:
				return $this->y;
			case 2:
				return $this->z;
			case 3:
				return $this->w;
			default:
				throw new Exception('Invalid argument.');
		}
	}

	/**
	 * Return all coordinates of the vector.
	 *
	 * @return Coordinates as array.
	 */
	function getArray() {
		return array($this->x, $this->y, $this->z, $this->w);
	}

	/**
	 * Return first coordinate of the vector.
	 *
	 * @return Requested value of the vector.
	 */
	function x() {
		return $this->x;
	}

	/**
	 * Return second coordinate of the vector.
	 *
	 * @return Requested value of the vector.
	 */
	function y() {
		return $this->y;
	}

	/**
	 * Return third coordinate of the vector.
	 *
	 * @return Requested value of the vector.
	 */
	function z() {
		return $this->z;
	}

	/**
	 * Return fourth coordinate of the vector.
	 *
	 * @return Requested value of the vector.
	 */
	function w() {
		return $this->w;
	}

	/**
	 * Length of this vector.
	 *
	 * @return Length of this vector.
	 */
	function length() {
		return sqrt(  $this->x * $this->x
					+ $this->y * $this->y
					+ $this->z * $this->z
					+ $this->w * $this->w);
	}

	/**
	 * Sum of this and a second vector.
	 *
	 * @param $v Second vector.
	 * @return Resulting vector.
	 */
	function add(Vector4 $v) {
		return new Vector4(	$this->x + $v->x,
							$this->y + $v->y,
							$this->z + $v->z,
							$this->w + $v->w);
	}

	/**
	 * Difference of this and a second vector.
	 *
	 * @param $v Second vector.
	 * @return Resulting vector.
	 */
	function sub(Vector4 $v) {
		return new Vector4(	$this->x - $v->x,
							$this->y - $v->y,
							$this->z - $v->z,
							$this->w - $v->w);
	}

	/**
	 * Scalar product of this and a second vector.
	 *
	 * @param $v Second vector.
	 * @return Result as floating point number.
	 */
	function mult(Vector4 $v) {
		return	  $this->x * $v->x
				+ $this->y * $v->y
				+ $this->z * $v->z
				+ $this->w * $v->w;
	}

	/**
	 * Cross product of this and a second vector. It uses only the first three
	 * coordinates.
	 *
	 * @param $v Second vector.
	 * @return Resulting vector.
	 */
	function cross(Vector4 $v) {
		return new Vector4(	$this->y * $v->z - $this->z * $v->y,
							$this->z * $v->x - $this->x * $v->z,
							$this->x * $v->y - $this->y * $v->x,
							1.0);
	}

	/**
	 * Product of this and a scalar.
	 *
	 * @param $s Scalar.
	 * @return Resulting vector.
	 */
	function multScalar($s) {
		return new Vector4(	$this->x * $s,
							$this->y * $s,
							$this->z * $s,
							$this->w * $s);
	}

	/**
	 * Divide this vector by it's length to make it unit length.
	 */
	function normalize() {
		$length = $this->length();
		$this->x /= $length;
		$this->y /= $length;
		$this->z /= $length;
		$this->w /= $length;
	}

	/**
	 * Return the vector as human-readable string representation.
	 *
	 * @return Vector representation as string.
	 */
	function __toString() {
		$result = "/\t".$this->x."\t\\\n";
		$result .= "|\t".$this->y."\t|\n";
		$result .= "|\t".$this->z."\t|\n";
		$result .= "\\\t".$this->w."\t/\n";
		return $result;
	}

	/**
	 * First coordinate of the vector.
	 */
	protected $x;

	/**
	 * Second coordinate of the vector.
	 */
	protected $y;

	/**
	 * Third coordinate of the vector.
	 */
	protected $z;

	/**
	 * Fourth coordinate of the vector.
	 */
	protected $w;
}
?>
