<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');

/**
 * Class for raster polygons.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2009-01-07
 */
class Rasterization extends Filter {

	/**
	 * This class accepts a Data2D object as input and generates a Bitmap object
	 * as output.
	 */
	public function __construct() {
		parent::__construct('Data2D', 'Bitmap');
	}

	/**
	 * Transform all vertices from the scene.
	 *
	 * @param $data 2D polygon data.
	 */
	public function process(& $data) {
		$bitmap = new Bitmap;
		$bitmap->width = 800;
		$bitmap->height = 600;
		$centerX = $bitmap->width / 2;
		$centerY = $bitmap->height / 2;

		// Draw polygons.
		foreach($data->data as $polygon) {
			if(!is_array($polygon)) {
				throw new Exception('Invalid polygon.');
			}

			$num = count($polygon);
			if($num == 3) {
				$scanlines = array();
				
				// First compute edges.
				for($i = 0; $i < 3; $i++) {
					
					// Make sure $srcY < $dstY.
					if($polygon[$i][0]->y() < $polygon[($i + 1) % 3][0]->y()) {
						$srcX = round($polygon[$i][0]->x()) + $centerX;
						$srcY = round($polygon[$i][0]->y()) + $centerY;
						$srcZ = $polygon[$i][0]->z();
						$srcColor = $polygon[$i][1];

						$dstX = round($polygon[($i + 1) % 3][0]->x()) + $centerX;
						$dstY = round($polygon[($i + 1) % 3][0]->y()) + $centerY;
						$dstZ = $polygon[($i + 1) % 3][0]->z();
						$dstColor = $polygon[($i + 1) % 3][1];
					} else {
						$srcX = round($polygon[($i + 1) % 3][0]->x()) + $centerX;
						$srcY = round($polygon[($i + 1) % 3][0]->y()) + $centerY;
						$srcZ = $polygon[($i + 1) % 3][0]->z();
						$srcColor = $polygon[($i + 1) % 3][1];

						$dstX = round($polygon[$i][0]->x()) + $centerX;
						$dstY = round($polygon[$i][0]->y()) + $centerY;
						$dstZ = $polygon[$i][0]->z();
						$dstColor = $polygon[$i][1];
					}

					// Discard horizontal lines.
					$deltaY = $dstY - $srcY;
					if($deltaY == 0.0) {
						continue;
					}

					// Compute edge.
					$x = $srcX;
					$z = $srcZ;
					$r = $srcColor->x() * 255.0;
					$g = $srcColor->y() * 255.0;
					$b = $srcColor->z() * 255.0;
					$mx = ($dstX - $srcX) / $deltaY;
					$mz = ($dstZ - $srcZ) / $deltaY;
					$mr = ($dstColor->x() - $srcColor->x()) * 255.0 / $deltaY;
					$mg = ($dstColor->y() - $srcColor->y()) * 255.0 / $deltaY;
					$mb = ($dstColor->z() - $srcColor->z()) * 255.0 / $deltaY;
					for($y = $srcY; $y < $dstY; $y++) {
						$xi = round($x);
						$ri = round($r);
						$gi = round($g);
						$bi = round($b);
						if(!isset($scanlines[$y]['left'])) {
							$scanlines[$y]['left']['x'] = $xi;
							$scanlines[$y]['left']['z'] = $z;
							$scanlines[$y]['left']['r'] = $ri;
							$scanlines[$y]['left']['g'] = $gi;
							$scanlines[$y]['left']['b'] = $bi;
							$scanlines[$y]['right']['x'] = $xi;
							$scanlines[$y]['right']['z'] = $z;
							$scanlines[$y]['right']['r'] = $ri;
							$scanlines[$y]['right']['g'] = $gi;
							$scanlines[$y]['right']['b'] = $bi;
						} else {
							if($xi < $scanlines[$y]['left']['x']) {
								$scanlines[$y]['left']['x'] = $xi;
								$scanlines[$y]['left']['z'] = $z;
								$scanlines[$y]['left']['r'] = $ri;
								$scanlines[$y]['left']['g'] = $gi;
								$scanlines[$y]['left']['b'] = $bi;
							} else if($xi > $scanlines[$y]['right']['x']) {
								$scanlines[$y]['right']['x'] = $xi;
								$scanlines[$y]['right']['z'] = $z;
								$scanlines[$y]['right']['r'] = $ri;
								$scanlines[$y]['right']['g'] = $gi;
								$scanlines[$y]['right']['b'] = $bi;
							}
						}
						$x += $mx;
						$z += $mz;
						$r += $mr;
						$g += $mg;
						$b += $mb;
					}
				}

				// Second fill polygon scanlines.
				foreach($scanlines as $y => $line) {
					$deltaX = $line['right']['x'] - $line['left']['x'];
					if($deltaX == 0.0) {
						continue;
					}

					$z = $line['left']['z'];
					$mz = ($line['right']['z'] - $line['left']['z']) / $deltaX;

					$r = $line['left']['r'];
					$g = $line['left']['g'];
					$b = $line['left']['b'];
					$mr = ($line['right']['r'] - $line['left']['r']) / $deltaX;
					$mg = ($line['right']['g'] - $line['left']['g']) / $deltaX;
					$mb = ($line['right']['b'] - $line['left']['b']) / $deltaX;
					for($x = $line['left']['x']; $x <= $line['right']['x']; $x++) {
						$index = $bitmap->width * $y + $x;
						if(!isset($bitmap->depth[$index]) || $z > $bitmap->depth[$index]) {
							$bitmap->color[$index] = (round($r) << 24)
													| (round($g) << 16)
													| (round($b) << 8)
													| 0x000000FF;
							$bitmap->depth[$index] = $z;
						}
						$z += $mz;
						$r += $mr;
						$g += $mg;
						$b += $mb;
					}
				}
			} else {
				throw new Exception('Only triangles are supported.');
			}
		}

		$this->send($bitmap);
	}
}
?>
