<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');
require_once('MathTransformation.php');
require_once('MathProjection.php');

/**
 * Class for transforming a scene from object space to screen space.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-05
 */
class Renderer extends Filter {

	/**
	 * This class accepts a Data3D object as input and generates a Data2D object
	 * as output.
	 */
	public function __construct() {
		parent::__construct('Data3D', 'Data2D');
		$this->modelViewMatrix = new Transformation;
		$this->projectionMatrix = new Projection;
		$this->projectionMatrix->makeOrthographic(-5.0, 5.0, -5.0, 5.0, 1.0, 100.0);
	}

	/**
	 * Transform all vertices from the scene.
	 *
	 * @param $data 3D scene data.
	 */
	public function process(& $data) {
		$m = new Matrix4x4;
		$m = $m->mult($this->projectionMatrix);
		$m = $m->mult($this->modelViewMatrix);

		$polygons = new Data2D;
		
		foreach($data->faces as $face) {
			if(!($face instanceof Face)) {
				throw new Exception('Invalid face.');
			}
			$num = count($face->vertices);
			if($num >= 2 || $num <= 4) {
				$polygon = array();
				$v = array();
				$maxZ = 0.0;
				for($i = 0; $i < $num; $i++) {
					$v[$i] = $m->multVector($face->vertices[$i]);
					$polygon[] = array($v[$i], $face->colors[$i]);
					$maxZ = max($v[$i]->z(), $maxZ);
				}
				$index = $maxZ * 1E6;
				while(isset($polygons->data[$index])) {
					$index++;
				}
				$polygons->data[$index] = $polygon;
			} else {
				throw new Exception('Face has invalid number of vertices.');
			}
		}
		
		ksort($polygons->data, SORT_NUMERIC);
		$this->send($polygons);
	}

	/**
	 * Return the model-view matrix.
	 *
	 * @return Matrix used for converting vectors from object space to camera 
	 * space.
	 */
	public function getModelView() {
		return $this->modelViewMatrix;
	}

	/**
	 * Return the projection matrix.
	 *
	 * @return Matrix used for projecting vectors from camera space to screen 
	 * space.
	 */
	public function getProjection() {
		return $this->projectionMatrix;
	}

	/**
	 * Set a new model-view matrix.
	 *
	 * @param $m Matrix used for converting vectors from object space to camera 
	 * space.
	 */
	public function setModelView(Matrix4x4 $m) {
		$this->modelViewMatrix = $m;
	}

	/**
	 * Set a new projection matrix.
	 *
	 * @param $m Matrix used for projecting vectors from camera space to screen 
	 * space.
	 */
	public function setProjection(Matrix4x4 $m) {
		$this->projectionMatrix = $m;
	}

	/**
	 * Matrix used for converting vectors from object space to camera space.
	 */
	protected $modelViewMatrix;

	/**
	 * Matrix used for projecting vectors from camera space to screen space.
	 */
	protected $projectionMatrix;
}
?>
